function Solid_PostProcessing
close all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% some useful MatLab postprocessing functions for trusses
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%input data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%foldername  =  'plane_strain_nearly_incompressible_strip';
%foldername  =  'DYN_plane_strain_nearly_incompressible_strip';
%foldername  =  'DYN_bending_column';
%foldername  =  'DYN_bending_column_fine_mesh';
%foldername  =  'DYN_twisting_column';
%foldername  =  'DYN_twisting_column_fine_mesh';
%foldername  =  'DYN_rope_3Dsolid';
%foldername  =  'DYN_rope_3Dsolid_implicit';
%foldername  =  'DYN_L_block_solid';
%foldername  =  'DYN_L_block_solid_fine_mesh';
foldername  =  'DYN_L_block_solid_VERY_fine_mesh';
%foldername  =  'DYN_simple_two_dimensional_example';
%foldername  =  'DYN_simple_patch_test';
%foldername  =  'DYN_swinging_square_coarse';
%foldername  =  'DYN_swinging_square_medium';
%foldername  =  'DYN_swinging_square_fine';
%foldername  =  'DYN_swinging_square_very_fine';
 
%incremental output for Paraview PostProcessing
delta_incrm=50;
%generation of VTK files (YES=1; NO=0)
VTK_flag=0;
%scale factor for Paraview postprocessing
scale_factor=1;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%perform actions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
perform_tasks(foldername,delta_incrm,VTK_flag,scale_factor)

end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%List of functions used above
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function perform_tasks(foldername,delta,VTK_flag,scale_factor)
fprintf('   job name: "%s" \n', foldername);
%add directory to path and create VTK subfolder
[max,folder,VTK_subfolder]=add_path(foldername,delta);


% FIGURE 1: plot energy evolution (figure 1)
energy_plot(folder,foldername,max*delta);

% FIGUREs 2 and 3: plot (time vs coordinate & coordinate vs force) of a given degree of
% freedom based on _FLAGOUT.TXT (figures 2 and 3)
flagout(folder,foldername);

%generate VTK outputs
if VTK_flag
   generate_VTK_files(max,delta,VTK_subfolder,foldername,scale_factor);
end


end

function ParaviewPostprocessor_new(FEM,GEOM,output_filename,initial_mesh,scale_factor)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Extraction and preparaton of data to be plotted in VTK format
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%2D quad and 3D hexa
FEM.postproc.N        =  eye(FEM.mesh.n_nodes_elem);
n_nodes               =  2^GEOM.ndime*FEM.mesh.nelem;
connectivity          =  reshape((1:n_nodes)',2^GEOM.ndime,[]);
%--------------------------------------------------------------------------
%  Initialise postprocessing variables 
%--------------------------------------------------------------------------
x_solid = zeros(GEOM.ndime,n_nodes);
X_solid = zeros(GEOM.ndime,n_nodes);
%--------------------------------------------------------------------------
%  Loop over elements 
%--------------------------------------------------------------------------
final =  0;
for ielem=1:FEM.mesh.nelem
    %----------------------------------------------------------------------
    % x and X of the postprocessing mesh   
    %----------------------------------------------------------------------
    xelem = zeros(GEOM.ndime,2^GEOM.ndime);
    Xelem = zeros(GEOM.ndime,2^GEOM.ndime);
    for inode=1:2^GEOM.ndime
        if initial_mesh
           xelem(:,inode) = GEOM.x0(:,FEM.mesh.connectivity(:,ielem))*FEM.postproc.N(:,inode);
        else
           xelem(:,inode) = GEOM.x(:,FEM.mesh.connectivity(:,ielem))*FEM.postproc.N(:,inode); 
        end
        Xelem(:,inode) = GEOM.x0(:,FEM.mesh.connectivity(:,ielem))*FEM.postproc.N(:,inode);        
    end 
    %
    initial                       =  final + 1;
    final                         =  final + 2^GEOM.ndime;
    %
    x_solid(:,initial:final)      =  Xelem+scale_factor*(xelem-Xelem);
    X_solid(:,initial:final)      =  Xelem;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  vtkwrite(filename,'unstructured_grid',x,y,z,'vectors',title1,u,v,w,'scalars',
%  title2,r) writes a 3D unstructured grid that contains both vector and scalar values.
%  x,y,z,u,v,w,r must all be the same size and contain the corresponding
%  positon, vector and scalar values.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if GEOM.ndime==2
   x_solid(3,:) = 0;
   ux =  reshape(x_solid(1,:) - X_solid(1,:),[],1);
   uy =  reshape(x_solid(2,:) - X_solid(2,:),[],1);
   VTKPlot_new(output_filename,'unstructured_grid',x_solid',connectivity',...
               'scalars','ux',ux,'scalars','uy',uy); 
else  
    ux =  reshape(x_solid(1,:) - X_solid(1,:),[],1);
    uy =  reshape(x_solid(2,:) - X_solid(2,:),[],1);
    uz =  reshape(x_solid(3,:) - X_solid(3,:),[],1);
    VTKPlot_new(output_filename,'unstructured_grid',x_solid',connectivity',...
                'scalars','ux',ux,'scalars','uy',uy,'scalars','uz',uz); 
end    

end

function VTKPlot_new(filename,dataType,varargin)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% writing data into VTK format
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% VTKWRITE Writes 3D Matlab array into VTK file format.
%  vtkwrite(filename,'structured_grid',x,y,z,'vectors',title,u,v,w) writes
%  a structured 3D vector data into VTK file, with name specified by the string
%  filename. (u,v,w) are the vector components at the points (x,y,z). x,y,z
%  should be 3-D matrices like those generated by meshgrid, where
%  point(ijk) is specified by x(i,j,k), y(i,j,k) and z(i,j,k).
%  The matrices x,y,z,u,v,w must all be the same size and contain
%  corrresponding position and vector component. The string title specifies
%  the name of the vector field to be saved. 
%
%  vtkwrite(filename,'structured_grid',x,y,z,'scalars',title,r) writes a 3D
%  scalar data into VTK file whose name is specified by the string
%  filename. r is the scalar value at the points (x,y,z). The matrices
%  x,y,z,r must all be the same size and contain the corresponding position
%  and scalar values. 
%
%  vtkwrite(filename,'structured_grid',x,y,z,'vectors',title1,u,v,w,'scalars',
%  title2,r) writes a 3D structured grid that contains both vector and scalar values.
%  x,y,z,u,v,w,r must all be the same size and contain the corresponding
%  positon, vector and scalar values.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  vtkwrite(filename,'unstructured_grid',x,y,z,'vectors',title1,u,v,w,'scalars',
%  title2,r) writes a 3D unstructured grid that contains both vector and scalar values.
%  x,y,z,u,v,w,r must all be the same size and contain the corresponding
%  positon, vector and scalar values.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  vtkwrite(filename,'polydata','lines',x,y,z) exports a 3D line where
%  x,y,z are coordinates of the points that make the line. x,y,z are
%  vectors containing the coordinates of points of the line, where point(n)
%  is specified by x(n),y(n) and z(n).
%
%  vtkwrite(filename,'polydata','lines',x,y,z,'Precision',n) allows you to
%  specify precision of the exported number up to n digits after decimal
%  point. Default precision is 3 digits. 
%
%  vtkwrite(filename,'polydata','triangle',x,y,z,tri) exports a list of
%  triangles where x,y,z are the coordinates of the points and tri is an
%  m*3 matrix whose rows denote the points of the individual triangles.
%
%  vtkwrite(filename,'polydata','tetrahedron',x,y,z,tetra) exports a list
%  of tetrahedrons where x,y,z are the coordinates of the points
%  and tetra is an m*4 matrix whose rows denote the points of individual
%  tetrahedrons. 
%  
%  vtkwrite('execute','polydata','lines',x,y,z) will save data with default
%  filename ''matlab_export.vtk' and automatically loads data into
%  ParaView. 
%  
%  Version 2.2.1
%  Chaoyuan Yeh, 2015
%  Codes are modified from William Thielicke and David Gingras's submission.    

if strcmpi(filename,'execute'), filename = 'matlab_export.vtk'; end
fid = fopen(filename, 'w'); 
% VTK files contain five major parts
% 1. VTK DataFile Version
fprintf(fid, '# vtk DataFile Version 2.0\n');
% 2. Title
fprintf(fid, 'VTK from Matlab\n');
fprintf(fid, 'ASCII\n');
%--------------------------------------------------------------------------
% Preprocess
%--------------------------------------------------------------------------
data.dim         = numel(varargin{1})/length(varargin{1});
data.nodes       = length(varargin{1});
data.elem        = size(varargin{2},1);
data.nperelement = size(varargin{2},2);

if data.dim == 2 && data.nperelement == 3,  data.shape = '1-triangular';   verb1 = '%0.3f %0.3f\n'; verb2 = '%d %d %d %d\n'; b = 5;  end
if data.dim == 2 && data.nperelement == 4,  data.shape = '1-quadrilateral';verb1 = '%0.3f %0.3f\n'; verb2 = '%d %d %d %d %d\n'; b = 9;  end
if data.dim == 2 && data.nperelement == 6,  data.shape = '2-triangular';   verb1 = '%0.3f %0.3f\n'; verb2 = '%d %d %d %d %d %d %d\n'; b = 22; end
if data.dim == 2 && data.nperelement == 8,  data.shape = '2-quadrilateral';verb1 = '%0.3f %0.3f\n'; verb2 = '%d %d %d %d %d %d %d %d %d\n'; b = 23; end
if data.dim == 3 && data.nperelement == 4,  data.shape = '1-tetrahedron';  verb1 = '%0.3f %0.3f %0.3f\n'; verb2 = '%d %d %d %d %d\n'; b = 10; end
if data.dim == 3 && data.nperelement == 8,  data.shape = '1-hexahedron';   verb1 = '%0.3f %0.3f %0.3f\n'; verb2 = '%d %d %d %d %d %d %d %d %d\n'; b = 12; end
if data.dim == 3 && data.nperelement == 4,  data.shape = '1-quadrilateral';verb1 = '%0.3f %0.3f %0.3f\n'; verb2 = '%d %d %d %d %d\n'; b = 9;  end
if data.dim == 3 && data.nperelement == 10, data.shape = '2-tetrahedron';  verb1 = '%0.3f %0.3f %0.3f\n'; verb2 = '%d %d %d %d %d %d %d %d %d %d %d\n'; b = 24; end
if data.dim == 3 && data.nperelement == 20, data.shape = '2-hexahedron';   verb1 = '%0.3f %0.3f %0.3f\n'; verb2 = '%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d\n'; b = 25; end
if data.dim == 3 && data.nperelement == 27, data.shape = '2bi-hexahedron';   verb1 = '%0.3f %0.3f %0.3f\n'; verb2 = '%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d  %d  %d  %d  %d  %d  %d  %d\n'; b = 28; end
%--------------------------------------------------------------------------
% Mesh type 
%--------------------------------------------------------------------------
fprintf(fid, '\nDATASET UNSTRUCTURED_GRID\n');
%--------------------------------------------------------------------------
% Write the coordinates of the nodes
%--------------------------------------------------------------------------
fprintf(fid, ['POINTS ' num2str(data.nodes) ' float\n']);
fprintf(fid,verb1,varargin{1}');
%--------------------------------------------------------------------------
% Write type of element and connectivity
%--------------------------------------------------------------------------
fprintf(fid, ['\nCELLS ' num2str(data.elem) ' ' num2str(data.elem*(data.nperelement+1)) '\n']);
fprintf(fid,verb2,[data.nperelement*ones(data.elem,1),varargin{2}-1]');

fprintf(fid, ['\nCELL_TYPES ' num2str(data.elem) '\n']);
fprintf(fid,'%d\n',b*ones(data.elem,1)');
%--------------------------------------------------------------------------
% Write the data at every node, either if it is scalar or vector
%--------------------------------------------------------------------------
fprintf(fid, ['\nPOINT_DATA ' num2str(data.nodes)]);
vidx = find(strcmpi(varargin,'VECTORS'));
sidx = find(strcmpi(varargin,'SCALARS'));
if vidx~=0
    for ii = 1:length(vidx)
        title = varargin{vidx(ii)+1};
        % Data enteries begin with a keyword specifying data type
        % and numeric format.
        fprintf(fid, ['\nVECTORS ', title,' float\n']);
        fwrite(fid, [reshape(varargin{vidx(ii)+2},1,n_points);...
        reshape(varargin{vidx(ii)+3},1,n_points);...
        reshape(varargin{vidx(ii)+4},1,n_points)],'float','b');
    end
end
if sidx~=0
    for ii = 1:length(sidx)
        title = varargin{sidx(ii)+1};
        fprintf(fid, ['\nSCALARS ', title,' float\n']);
        fprintf(fid, 'LOOKUP_TABLE default\n');
        val = varargin{sidx(ii)+2};
        fprintf(fid,'%0.4f\n',val);
    end
end

fclose(fid);
if strcmpi(filename,'matlab_export.vtk')
    switch computer
        case {'PCWIN','PCWIN64'}
            !paraview.exe --data='matlab_export.vtk' &
            % Exclamation point character is a shell escape, the rest of the
            % input line will be sent to operating system. It can not take
            % variables, though. The & at the end of line will return control to 
            % Matlab even when the outside process is still running. 
        case {'GLNXA64','MACI64'}
            !paraview --data='matlab_export.vtk' &
    end
end
end

function [last,folder,VTK_subfolder]=add_path(foldername,delta)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% add relevant directory to path
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%move to relevant directory
if isunix()
    dirsep =  '/';    
else   
    dirsep =  '\';
end
%current file
basedir = mfilename('fullpath');
%current folder
basedir = fileparts(basedir);
%folder with relevant example
folder=[basedir dirsep 'job_folder' dirsep foldername];
%add folder to path
addpath(folder)
%create a VTK SubFolder to save VTK output files
VTK_subfolder=[folder dirsep 'VTK_output'];
%create the SubFolder if it doesn't exist already.
if ~exist(VTK_subfolder, 'dir')
    mkdir(VTK_subfolder);
end

%count files to be read
i = 0;
reading = 1;
while reading
    i = i+1; 
    filename  =  [foldername '-increment_' num2str(delta*i) '-RESTART.MAT'];
    if exist(filename,'file')
    else
        last = i-1;
        break;
    end
end

end

function generate_VTK_files(max,delta,VTK_subfolder,foldername,scale_factor)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data and generate VTK files
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('   Generating VTK files\n');
%
for i=1:max   
    %define file name
    filename  =  [foldername '-increment_' num2str(delta*i) '-RESTART.MAT'];
    %load file name
    load(filename);
    if isunix()
        dirsep =  '/';
    else
        dirsep =  '\';
    end
    %
    if i==1
       output_filename=[VTK_subfolder dirsep foldername num2str(0) '.vtk'];
       ParaviewPostprocessor_new(FEM,GEOM,output_filename,1,scale_factor);
    end
    output_filename=[VTK_subfolder dirsep foldername num2str(i) '.vtk'];
    ParaviewPostprocessor_new(FEM,GEOM,output_filename,0,scale_factor);
    
end
end

function energy_plot(folder,foldername,i)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plot energy evolution based on a given output file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%define file name
filename  =  [foldername '-increment_' num2str(1*i) '-RESTART.MAT'];
%load file name
load(filename);

%plot energy evolution
dt=CON.DYNAMICS.dt;
t=dt*[1:size(GEOM.Energy,2)];
h=figure;movegui(gcf,'northwest');
% plot(t,GEOM.Energy,   'bo-',t,GEOM.IntEnergy,'rs--',t,GEOM.ExtEnergy,'kd:',t,GEOM.Kinetic,  'gx-.',...
%      'LineWidth', 2, 'MarkerIndices', 1:5:length(t));
plot(t,GEOM.Energy, 'k-',t,GEOM.IntEnergy,'k--',t,GEOM.ExtEnergy,'ks:',t,GEOM.Kinetic,  'k-.*',...
     'LineWidth', 1, 'MarkerIndices', 1:50:length(t));
legend('Total Energy','Internal Energy','External Work','Kinetic Energy');
xlabel('time (s)');ylabel('Energy (J)');
grid on

%send to PDF file
if isunix()
    dirsep =  '/';    
else   
    dirsep =  '\';
end
Fig_name=[folder dirsep 'Energy_plot'];
PDF_plot(Fig_name);

end

function PDF_plot(Fig_name)
set(gcf,'PaperUnits','inches');
set(gcf,'PaperPositionMode','auto');
set(gcf,'PaperSize',[5.5 4.5]);
print(gcf, '-dpdf', Fig_name,'-bestfit');
end

function flagout(folder,foldername)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plot time history of a given degree of freedom based on _FLAGOUT.TXT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%plot time history of a given degree of freedom (from _FLAGOUT.TXT)
filename2 = [foldername '-OUTPUT_FLAGOUT.TXT'];
fid = fopen(filename2,'r+');
format = ['%g %g %g %g'];
info = fscanf(fid,format,[4,inf]);
fclose(fid);
incr=info(1,1:end);
x   =info(2,1:end);
f   =info(3,1:end);

%Increment vs Coordinate
h=figure;movegui(gcf,'north');
plot(incr,x,'ko-','LineWidth', 2, 'MarkerIndices', 1:1:length(x));
xlabel('time increment');ylabel('coordinate');
grid on;

%Coordinate vs Force applied
h=figure;movegui(gcf,'northeast');
plot(x,f,'ks-','LineWidth', 2, 'MarkerIndices', 1:1:length(x));
xlabel('Coordinate');ylabel('Force');
grid on;

% %temporal fix to plot "simple patch test example"
% filetemp= [foldername '-OUTPUT_FLAGOUT_horizontal_displacement.TXT'];
% fidtemp = fopen(filetemp,'r+');
% format = ['%g %g %g %g'];
% info = fscanf(fidtemp,format,[4,inf]);
% fclose(fidtemp);
% xtemp   =info(2,1:end);
% h=figure;movegui(gcf,'north');
% plot(incr,xtemp,'ko-',incr,x,'ks-','LineWidth', 1, 'MarkerIndices', 1:1:length(x));
% legend('x_1','x_2');
% xlabel('time increment');ylabel('coordinate');
% grid on;
% %send to PDF file
% if isunix()
%     dirsep =  '/';    
% else   
%     dirsep =  '\';
% end
% Fig_name=[folder dirsep 'Displacement_plot'];
% PDF_plot(Fig_name);


end

