%--------------------------------------------------------------------------
% Computes the element vector of global internal forces and the tangent
% stiffness matrix. 
%--------------------------------------------------------------------------
function Energy = element_energy(FEM,xlocal,x0local,...
                                 Ve,QUADRATURE,properties,dim,...
                                 matyp,KINEMATICS)
%--------------------------------------------------------------------------
% Computes initial and current gradients of shape functions and various 
% strain measures at all the Gauss points of the element.
%--------------------------------------------------------------------------
KINEMATICS = gradients(xlocal,x0local,FEM.interpolation.element.DN_chi,...
             QUADRATURE,KINEMATICS);
%--------------------------------------------------------------------------
% Computes element volume in deformed configuration. 
%--------------------------------------------------------------------------
ve = 0;
for igauss=1:QUADRATURE.ngauss
    JW = KINEMATICS.Jx_chi(igauss)*QUADRATURE.W(igauss);
    ve = ve + JW;
end
%--------------------------------------------------------------------------
% Gauss quadrature integration loop.
%--------------------------------------------------------------------------
Energy         =  0;
for igauss=1:QUADRATURE.ngauss
    %----------------------------------------------------------------------
    % Extract kinematics at the particular Gauss point.
    %----------------------------------------------------------------------
    kinematics_gauss = kinematics_gauss_point(KINEMATICS,igauss);     
    %----------------------------------------------------------------------
    % Obtain stresses (for incompressible or nearly incompressible, 
    % only deviatoric component) and internal variables in plasticity.
    %----------------------------------------------------------------------    
    W = Energy_type_selection(kinematics_gauss,properties,dim,matyp);                                         
    %----------------------------------------------------------------------
    % Compute numerical integration multipliers.
    %----------------------------------------------------------------------
    JW = kinematics_gauss.Jx_chi*QUADRATURE.W(igauss)*...
         thickness_plane_stress(properties,kinematics_gauss.J,matyp);
    %----------------------------------------------------------------------
    % Compute equivalent (internal) force vector.
    %----------------------------------------------------------------------
    Energy      =  Energy + W*JW;
end

%----------------------------------------------------------------------
% Add pressure contribution to energy.
%----------------------------------------------------------------------
Energy = mean_dilatation_energy_addition(Energy,matyp,properties,ve,Ve);

end

