%--------------------------------------------------------------------------
% Computes the element vector of global internal forces and the tangent
% stiffness matrix. 
%--------------------------------------------------------------------------
function [indexi,indexj,global_mass,counter] = ...
          element_mass_matrix(FEM,xlocal,x0local,...
          element_connectivity,QUADRATURE,properties,dim,...
          matyp,counter,KINEMATICS,indexi,indexj,global_mass)

%--------------------------------------------------------------------------
% Computes initial and current gradients of shape functions and various 
% strain measures at all the Gauss points of the element.
%--------------------------------------------------------------------------
KINEMATICS = gradients(xlocal,x0local,FEM.interpolation.element.DN_chi,...
             QUADRATURE,KINEMATICS);
%--------------------------------------------------------------------------
% Gauss quadrature integration loop. 
%--------------------------------------------------------------------------
for igauss=1:QUADRATURE.ngauss
    %----------------------------------------------------------------------
    % Extract kinematics at the particular Gauss point.
    %----------------------------------------------------------------------
    kinematics_gauss = kinematics_gauss_point(KINEMATICS,igauss);     
    %----------------------------------------------------------------------
    % Compute numerical integration multipliers.
    %----------------------------------------------------------------------
    JW = kinematics_gauss.Jx_chi*QUADRATURE.W(igauss)*...
         thickness_plane_stress(properties,kinematics_gauss.J,matyp);
    %----------------------------------------------------------------------
    % Compute conttribution (and extract relevant information for subsequent
    % assembly) of the constitutive term of the stiffness matrix.
    %----------------------------------------------------------------------
    [indexi,indexj,global_mass,counter] = ...
          mass_matrix(FEM,dim,element_connectivity,...
          FEM.interpolation.element.N(:,igauss),JW,properties(1),counter,indexi,indexj,...
          global_mass);
      
%     [indexi,indexj,global_stiffness,...
%      counter] = constitutive_matrix(FEM,dim,element_connectivity,...
%      kinematics_gauss,c,JW,counter,indexi,indexj,global_stiffness);
end
end

