%--------------------------------------------------------------------------
% Computes the element vector of global internal forces and the tangent
% stiffness matrix (3D truss).
%--------------------------------------------------------------------------
function [indexi,indexj,global_mass,counter] = ...
          element_mass_truss(properties,X_local,...
          element_connectivity,FEM,counter,indexi,indexj,...
          global_mass,GEOM)

density  = properties(1);       
area     = properties(4);  
%--------------------------------------------------------------------------
% Temporary variables.
%--------------------------------------------------------------------------
L       = norm(X_local(:,2) - X_local(:,1));  
V       = area*L;                             
%--------------------------------------------------------------------------
% Computation of the stiffness matrix.
%--------------------------------------------------------------------------
Mbb        = eye(3);            
Mass       = density*V*[[2/6*Mbb 1/6*Mbb];[1/6*Mbb  2/6*Mbb]];           
%--------------------------------------------------------------------------
% Sparse assembly of the stiffness matrix. 
%--------------------------------------------------------------------------
element_indexi               = reshape(FEM.mesh.dof_nodes(:,element_connectivity),[],1);
element_indexi               = repmat(element_indexi,1,6);
element_indexj               = element_indexi';
n_dofs_elem                  = (FEM.mesh.n_nodes_elem*GEOM.ndime)^2;
aux_vector                   = counter:counter+n_dofs_elem-1;
indexi(aux_vector)           = element_indexi(:);
indexj(aux_vector)           = element_indexj(:);
global_mass(aux_vector)      = Mass(:);
%--------------------------------------------------------------------------
% Global index for sparse assembly.
%--------------------------------------------------------------------------
counter = counter + n_dofs_elem;
end