%--------------------------------------------------------------------------
% Computes the element vector of global internal forces and the tangent
% stiffness matrix (3D truss).
%--------------------------------------------------------------------------
function [T_internal,PLAST,Cauchy] = ...
          explicit_element_force_truss(properties,x_local,X_local,...
                     FEM,PLAST,GEOM)
area  = properties(4);  
E     = properties(2);  
ty0   = properties(5);  
H     = properties(6);  
ep    = PLAST.ep;    
epbar = PLAST.epbar; 
%--------------------------------------------------------------------------
% Temporary variables.
%--------------------------------------------------------------------------
L       = norm(X_local(:,2) - X_local(:,1));  
dx      = x_local(:,2) - x_local(:,1);        
l       = norm(dx);                           
n       = dx/l;                               
V       = area*L;                             
lambda  = l/L;                                
epsilon = log(lambda);
%--------------------------------------------------------------------------
% Return mapping algorithm for trusses.
% - Trial stage.
%--------------------------------------------------------------------------
epsilon_trial = epsilon - ep;
tau_trial     = E*epsilon_trial;
%--------------------------------------------------------------------------
% - Check yield criterion.
%--------------------------------------------------------------------------
f = abs(tau_trial) - (ty0+H*epbar);  
%--------------------------------------------------------------------------
% - Return mapping algorithm.
%--------------------------------------------------------------------------
if f>0
   Dgamma          = f/(E + H); 
else
   Dgamma          = 0;         
end
Dep                = Dgamma*sign(tau_trial);  
tau                = tau_trial - E*Dep;       
ep                 = ep + Dep;                 
epbar              = epbar + Dgamma;          
%--------------------------------------------------------------------------
% Computation of the internal force vector.
%--------------------------------------------------------------------------
T          = tau*V/l;                  
Tb         = T*n;                         
T_internal = [-Tb;Tb];  
%--------------------------------------------------------------------------
% Storage of plastic variables.
%--------------------------------------------------------------------------
PLAST.ep    = ep;
PLAST.epbar = epbar;
%--------------------------------------------------------------------------
% Storage of stress for postprocessing purposes.
%--------------------------------------------------------------------------
J      = lambda^(1-2*properties(3));
Cauchy = tau/J;
end