%--------------------------------------------------------------------------
% Computes and assemble residual force vector and global tangent stiffness
% matrix except surface (line) element pressure contributions.
%--------------------------------------------------------------------------
function [Energy,K,U,Wext] = TotalEnergyComputation(GEOM,MAT,FEM,GLOBAL,QUADRATURE,...
                                                KINEMATICS,CON)    
%--------------------------------------------------------------------------
% Initialisation of the updated value of the internal variables.
%--------------------------------------------------------------------------
U  =  0;
%--------------------------------------------------------------------------
% Main element loop.
%--------------------------------------------------------------------------
for ielement=1:FEM.mesh.nelem
    %----------------------------------------------------------------------
    % Temporary variables associated with a particular element.
    %----------------------------------------------------------------------
    global_nodes    = FEM.mesh.connectivity(:,ielement);   
    material_number = MAT.matno(ielement);     
    matyp           = MAT.matyp(material_number);        
    properties      = MAT.props(:,material_number); 
    xlocal          = GEOM.x(:,global_nodes);                     
    x0local         = GEOM.x0(:,global_nodes);                       
    Ve              = GEOM.Ve(ielement);            
    %----------------------------------------------------------------------
    % Compute internal force and stiffness matrix for an element.
    %----------------------------------------------------------------------    
    switch FEM.mesh.element_type
      case 'truss2'
           U =  U  +  element_internal_energy_truss(properties,xlocal,x0local);
        otherwise
           U  = U +  element_energy(FEM,xlocal,x0local,Ve,QUADRATURE,properties,...
                         GEOM.ndime,matyp,KINEMATICS);                             
    end
end
%--------------------------------------------------------------------------
% Compute kinetic energy.
%--------------------------------------------------------------------------
if CON.DYNAMICS.time_integrator==1
   K   =  0.5*GEOM.velocity(:)'*(GLOBAL.Mass.*GEOM.velocity(:));    
else
   K   =  0.5*GEOM.velocity(:)'*(GLOBAL.Mass*GEOM.velocity(:));
end
%--------------------------------------------------------------------------
% Compute external force potential.
%--------------------------------------------------------------------------
Wext   =  CON.xlamb*GLOBAL.nominal_external_load'*(GEOM.x(:) - GEOM.x0(:));
%--------------------------------------------------------------------------
% Compute total energy (excluding external force potential).
%--------------------------------------------------------------------------
Energy    =  U + K - Wext;
end

 


 
