%--------------------------------------------------------------------------
% Computes and assemble mass matrix.
%--------------------------------------------------------------------------
function GLOBAL = consistent_mass_matrix_assembly(GEOM,MAT,FEM,GLOBAL,...
                                                     QUADRATURE,KINEMATICS)    

%--------------------------------------------------------------------------
% Recompute (increase) quadrature rules for full integration of mass matrix
% in the case of triangles and tetrahedra.
%--------------------------------------------------------------------------
if ~strcmp(FEM.mesh.element_type,'truss2')
   switch FEM.mesh.element_type
      case {'tria3','tria6','tetr4','tetr10'}        
           QUADRATURE = element_quadrature_mass_rules(FEM.mesh.element_type);
           FEM = shape_functions_iso_derivs_mass(QUADRATURE,FEM,GEOM.ndime);
   end
end
%--------------------------------------------------------------------------
% Pre-allocation memory to indexi, indexj and data for sparse assembly of
% the masss matrix.
%--------------------------------------------------------------------------
switch FEM.mesh.element_type
  case 'truss2'
     n_components = FEM.mesh.nelem*FEM.mesh.n_dofs_elem^2; 
    otherwise
     n_components =  ...
     FEM.mesh.nelem*(GEOM.ndime*FEM.mesh.n_nodes_elem^2)*QUADRATURE.ngauss;
end
%--------------------------------------------------------------------------
% Initialise counter for storing sparse information into 
% global mass matrix.
%--------------------------------------------------------------------------
counter          = 1;                                        
indexi           = zeros(n_components,1);
indexj           = zeros(n_components,1);
global_mass      = zeros(n_components,1); 
%
%--------------------------------------------------------------------------
% Main element loop.
%--------------------------------------------------------------------------
for ielement=1:FEM.mesh.nelem
    %----------------------------------------------------------------------
    % Temporary variables associated with a particular element.
    %----------------------------------------------------------------------
    global_nodes    = FEM.mesh.connectivity(:,ielement);   
    material_number = MAT.matno(ielement);     
    matyp           = MAT.matyp(material_number);        
    properties       = MAT.props(:,material_number); 
    xlocal          = GEOM.x(:,global_nodes);                     
    x0local         = GEOM.x0(:,global_nodes);                       
    %----------------------------------------------------------------------
    % Compute internal force and stiffness matrix for an element.
    %----------------------------------------------------------------------    
    switch FEM.mesh.element_type
      case 'truss2'
%        [T_internal,indexi,indexj,global_stiffness,counter,PLAST_element] = ...
%         element_force_and_stiffness_truss(properties,xlocal,x0local,...
%         global_nodes,FEM,PLAST_element,counter,indexi,indexj,...
%         global_stiffness,GEOM);
          [indexi,indexj,global_mass,counter] = ...
                                 element_mass_truss(properties,x0local,...
                                global_nodes,FEM,counter,indexi,indexj,...
                                global_mass,GEOM);    
        otherwise
       [indexi,indexj,global_mass,counter] = ...
                           element_mass_matrix(FEM,xlocal,x0local,...
                           global_nodes,QUADRATURE,properties,GEOM.ndime,...
                           matyp,counter,KINEMATICS,indexi,indexj,global_mass);
    end
end 
%--------------------------------------------------------------------------
% Global tangent stiffness matrix sparse assembly except pressure contributions. 
%--------------------------------------------------------------------------
GLOBAL.Mass = sparse(indexi,indexj,global_mass);                
end

 


 
