%--------------------------------------------------------------------------
% Update nodal forces due to external pressure 
% boundary face (line) contributions. 
%--------------------------------------------------------------------------
function GLOBAL = explicit_pressure_load_assembly(GEOM,FEM,GLOBAL,...
                 LOAD,QUADRATURE,xlamb)    
%--------------------------------------------------------------------------
% Initialise variables.
%--------------------------------------------------------------------------
GLOBAL.nominal_pressure = zeros(FEM.mesh.n_dofs,1);  
GLOBAL.R_pressure       = zeros(FEM.mesh.n_dofs,1);
GLOBAL.K_pressure       = sparse([],[],[],FEM.mesh.n_dofs,FEM.mesh.n_dofs);
%--------------------------------------------------------------------------
% Loop over all boundary (pressure load) elements.
%--------------------------------------------------------------------------
for ipressure=1:LOAD.n_pressure_loads    
    %----------------------------------------------------------------------
    % Intermediate variables associated to a particular element (ipressure).
    %----------------------------------------------------------------------
    element         = LOAD.pressure_element(ipressure);
    global_nodes    = FEM.mesh.connectivity_faces(:,element); 
    xlocal_boundary = GEOM.x(:,global_nodes);            
    %----------------------------------------------------------------------
    % Compute boundary (UNIT pressure load) force vector and stiffness matrix 
    % contribution for a boundary (UNIT pressure load) element.
    %----------------------------------------------------------------------    
    R_pressure_0 = explicit_pressure_element_load(xlocal_boundary,...
                     GEOM.ndime,QUADRATURE,FEM);
    %----------------------------------------------------------------------
    % Compute boundary (NOMINAL pressure load) force vector contribution 
    % for a boundary (NOMINAL pressure load) element.
    %----------------------------------------------------------------------
    nominal_pressure = LOAD.pressure(ipressure)*R_pressure_0; 
    %----------------------------------------------------------------------
    % Assemble boundary (NOMINAL pressure load) element force vector 
    % contribution into global force vector. 
    %----------------------------------------------------------------------   
    GLOBAL.nominal_pressure = force_vectors_assembly(nominal_pressure,...
    global_nodes,GLOBAL.nominal_pressure,FEM.mesh.dof_nodes);
end    
%--------------------------------------------------------------------------
% Add boundary (CURRENT pressure load) global force vector contribution 
% into Residual.
%--------------------------------------------------------------------------
GLOBAL.Residual = GLOBAL.Residual + xlamb*GLOBAL.nominal_pressure;
