function Mesh_generator_Lblock
close all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Block 1: Dimensions and discretisation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Ny  =  1; Nz  =  1; Nx  =  2;
Ly  =  3; Lz  =  3; Lx  =  6;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Block 1: Connectivity
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
connectivity_block1=get_connectivity(Nx,Ny,Nz);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Block 1: Creating coordinates
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Dx  =  Lx/Nx; Dy  =  Ly/Ny; Dz  =  Lz/Nz;

Y   =  (0:Dy:3);
Z   =  (0:Dz:3);
X   =  (6:-Dx:0);

X_  =  repmat(X,(size(Y,2)),(size(Z,2)));
X_  =  X_';
X_  =  X_(:);

Y_  =  repmat(Y,(size(X,2)),(size(Z,2)));
Y_  =  Y_(:);

Z_  =  repmat(Z,size(X,2)*size(Y,2),1);
Z_  =  Z_(:);
nodes_block1 =  [X_  Y_  Z_]';


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Block 2: Dimensions and discretisation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Ny  =  2; Nz  =  1; Nx  =  1;
Ly  =  7; Lz  =  3; Lx  =  3;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Block 2: Connectivity
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
connectivity_block2=get_connectivity(Nx,Ny,Nz);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Block 2: Creating coordinates
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Dx  =  Lx/Nx; Dy  =  Ly/Ny; Dz  =  Lz/Nz;

Y   =  (3:Dy:10);
Z   =  (0:Dz:3);
X   =  (3:-Dx:0);

X_  =  repmat(X,(size(Y,2)),(size(Z,2)));
X_  =  X_';
X_  =  X_(:);

Y_  =  repmat(Y,(size(X,2)),(size(Z,2)));
Y_  =  Y_(:);

Z_  =  repmat(Z,size(X,2)*size(Y,2),1);
Z_  =  Z_(:);
nodes_block2 =  [X_  Y_  Z_]';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Creating mirror nodes matrix
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
a=find(abs(nodes_block1(2,:)-3)<=1.0e-3); 
b=find((nodes_block1(1,:)-3)<1.0e-1);
list1=intersect(a,b);
x1=nodes_block1(:,list1);

list2=find(abs(nodes_block2(2,:)-3)<=1.0e-3);
x2=nodes_block2(:,list2);

mirror=zeros(size(list2,2),2);
count=0;
for i1=1:size(list1,2)
    for i2=1:size(list2,2)
        if norm(x1(:,i1)-x2(:,i2))<=1.0e-3
           count=count+1;
           mirror(count,1)=list1(i1);
           mirror(count,2)=list2(i2);
           break;
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Creating new nodal numbering of block 2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
new_numbering=zeros(size(nodes_block2,2),2);
new_numbering(:,1)=[1:size(nodes_block2,2)]';
count=size(nodes_block1,2);
for i2=1:size(nodes_block2,2)
    if any(list2(:)==i2)
       a=find(mirror(:,2)==i2);
       i1=mirror(a);
       new_numbering(i2,2)=i1;
    else
       count=count+1; 
       new_numbering(i2,2)=count;
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% print out nodes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
n1=size(nodes_block1,2);
n2=size(nodes_block2,2);
ncommon=size(list1,2);
no_nodes=n1+n2-ncommon;
NodesPrint = zeros(no_nodes,5);
NodesPrint(:,1)=[1:no_nodes]';
NodesPrint(1:n1,3:5)=nodes_block1';
a=find(new_numbering(:,2)>n1);
NodesPrint(n1+1:end,3:5)=nodes_block2(:,a)';

for inode=1:no_nodes
    format =  ['%4d \t %4d \t %-1.4E \t %-1.4E \t %-1.4E\n'];
    fprintf(format,NodesPrint(inode,:));
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Print out connectivity
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
e1=size(connectivity_block1,2);
e2=size(connectivity_block2,2);
e=e1+e2;
ConnectivityPrint  =  zeros(e,10);
ConnectivityPrint(:,1)  =  (1:e)';
ConnectivityPrint(:,2)  =  1;
ConnectivityPrint(1:e1,3:10)  =  connectivity_block1';
%
for ielem=1:size(connectivity_block2,2)
    for i=1:8
        node=connectivity_block2(i,ielem);
        connectivity_block2(i,ielem)=new_numbering(node,2); 
    end
end
ConnectivityPrint(e1+1:end,3:10)  =  connectivity_block2';
for ielem=1:e
    format =  ['%3d %3d %3d %3d %3d %3d %3d %3d %3d %3d\n'];
    fprintf(format,ConnectivityPrint(ielem,:));
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%plot nodal labels
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure(1);
for i=1:size(NodesPrint,1)
    x=NodesPrint(i,3);
    y=NodesPrint(i,4);
    z=NodesPrint(i,5);
    text(x,y,z,['\bullet' num2str(i)],'Fontsize',8);
end   
xlabel('x axis');ylabel('y axis');zlabel('z axis');
grid on;
view([1 1 1]);

end



function connectivity=get_connectivity(Nx,Ny,Nz)
connectivity     =  zeros(Nx*Ny*Nz,8);
ielem            =  1;

for iz=1:Nz
    for iy=1:Ny
        for ix=1:Nx
            connectivity(ielem,1)  =  ix + (iy - 1)*(Nx + 1) + (iz - 1)*(Nx + 1)*(Ny + 1);
            connectivity(ielem,2)  =  ix + 1 + (iy - 1)*(Nx + 1) + (iz - 1)*(Nx + 1)*(Ny + 1);
            connectivity(ielem,4)  =  ix + (iy - 0)*(Nx + 1) + (iz - 1)*(Nx + 1)*(Ny + 1);
            connectivity(ielem,3)  =  ix + 1 + (iy - 0)*(Nx + 1) + (iz - 1)*(Nx + 1)*(Ny + 1);
            connectivity(ielem,5)  =  ix + (iy - 1)*(Nx + 1) + (iz - 0)*(Nx + 1)*(Ny + 1);
            connectivity(ielem,6)  =  ix + 1 + (iy - 1)*(Nx + 1) + (iz - 0)*(Nx + 1)*(Ny + 1);
            connectivity(ielem,8)  =  ix + (iy - 0)*(Nx + 1) + (iz - 0)*(Nx + 1)*(Ny + 1);
            connectivity(ielem,7)  =  ix + 1 + (iy - 0)*(Nx + 1) + (iz - 0)*(Nx + 1)*(Ny + 1);
            ielem  =  ielem + 1;
        end
    end
end
connectivity  =  connectivity';
end
